@@ -1,4 +1,3 @@
-#include "utility.hh"
 #include "rec_channel.hh"
 #include <boost/lexical_cast.hpp>
 #include <boost/bind.hpp>
@@ -90,11 +89,16 @@ string doDumpCache(T begin, T end)
 template<typename T>
 string doWipeCache(T begin, T end)
 {
-  int count=0;
-  for(T i=begin; i != end; ++i)
+  int count=0, countNeg=0;
+  for(T i=begin; i != end; ++i) {
     count+=RC.doWipeCache(toCanonic("", *i));
+    string canon=toCanonic("", *i);
+    countNeg+=SyncRes::s_negcache.count(tie(canon));
+    pair<SyncRes::negcache_t::iterator, SyncRes::negcache_t::iterator> range=SyncRes::s_negcache.equal_range(tie(canon));
+    SyncRes::s_negcache.erase(range.first, range.second);
+  }
 
-  return "wiped "+lexical_cast<string>(count)+" records\n";
+  return "wiped "+lexical_cast<string>(count)+" records, "+lexical_cast<string>(countNeg)+" negative records\n";
 }
 
 template<typename T>
@@ -134,6 +138,11 @@ static uint64_t getUserTimeMsec()
 }
 #endif
 
+static uint64_t calculateUptime()
+{
+  return time(0) - g_stats.startupTime;
+}
+
 static string doCurrentQueries()
 {
   ostringstream ostr;
@@ -184,12 +193,21 @@ RecursorControlParser::RecursorControlParser()
 
   addGetStat("qa-latency", &g_stats.avgLatencyUsec);
   addGetStat("unexpected-packets", &g_stats.unexpectedCount);
+  addGetStat("case-mismatches", &g_stats.caseMismatchCount);
   addGetStat("spoof-prevents", &g_stats.spoofCount);
 
   addGetStat("nsset-invalidations", &g_stats.nsSetInvalidations);
 
   addGetStat("resource-limits", &g_stats.resourceLimits);
   addGetStat("dlg-only-drops", &SyncRes::s_nodelegated);
+  
+  addGetStat("shunted-queries", &g_stats.shunted);
+  addGetStat("noshunt-size", &g_stats.noShuntSize);
+  addGetStat("noshunt-expired", &g_stats.noShuntExpired);
+  addGetStat("noshunt-nomatch", &g_stats.noShuntNoMatch);
+  addGetStat("noshunt-cname", &g_stats.noShuntCNAME);
+  addGetStat("noshunt-wrong-question", &g_stats.noShuntWrongQuestion);
+  addGetStat("noshunt-wrong-type", &g_stats.noShuntWrongType);
 
   addGetStat("negcache-entries", boost::bind(&SyncRes::negcache_t::size, ref(SyncRes::s_negcache)));
   addGetStat("throttle-entries", boost::bind(&SyncRes::throttle_t::size, ref(SyncRes::s_throttle)));
@@ -205,6 +223,8 @@ RecursorControlParser::RecursorControlParser()
   addGetStat("unreachables", &SyncRes::s_unreachables);
   addGetStat("chain-resends", &g_stats.chainResends);
 
+  addGetStat("uptime", calculateUptime);
+
 #ifndef WIN32
   //  addGetStat("query-rate", getQueryRate);
   addGetStat("user-msec", getUserTimeMsec);
@@ -212,10 +232,29 @@ RecursorControlParser::RecursorControlParser()
 #endif
 }
 
-static void doExit()
+static void doExitGeneric(bool nicely)
 {
   L<<Logger::Error<<"Exiting on user request"<<endl;
-  _exit(1);
+  extern RecursorControlChannel s_rcc;
+  s_rcc.~RecursorControlChannel(); 
+
+  extern string s_pidfname;
+  if(!s_pidfname.empty()) 
+    unlink(s_pidfname.c_str()); // we can at least try..
+  if(nicely)
+    exit(1);
+  else
+    _exit(1);
+}
+
+static void doExit()
+{
+  doExitGeneric(false);
+}
+
+static void doExitNicely()
+{
+  doExitGeneric(true);
 }
 
 string doTopRemotes()
@@ -267,6 +306,12 @@ string RecursorControlParser::getAnswer(const string& question, RecursorControlP
     return "bye\n";
   }
 
+  if(cmd=="quit-nicely") {
+    *command=&doExitNicely;
+    return "bye nicely\n";
+  }
+
+
   if(cmd=="dump-cache") 
     return doDumpCache(begin, end);
 
@@ -276,6 +321,16 @@ string RecursorControlParser::getAnswer(const string& question, RecursorControlP
   if(cmd=="wipe-cache") 
     return doWipeCache(begin, end);
 
+  if(cmd=="reload-lua-script") 
+    return doReloadLuaScript(begin, end);
+
+  if(cmd=="unload-lua-script") {
+    vector<string> empty;
+    empty.push_back(string());
+    return doReloadLuaScript(empty.begin(), empty.end());
+  }
+
+
   if(cmd=="top-remotes")
     return doTopRemotes();
 
