@@ -41,7 +41,9 @@ using namespace std;
 int Utility::closesocket( Utility::sock_t socket )
 {
   int ret=::close(socket);
-  if(ret < 0)
+  if(ret < 0 && errno == ECONNRESET) // see ticket 192, odd BSD behaviour
+    return 0;
+  if(ret < 0) 
     throw AhuException("Error closing socket: "+stringerror());
   return ret;
 }
@@ -174,13 +176,6 @@ void Utility::srandom( unsigned int seed )
   ::srandom(seed);
 }
 
-// Compares two string, ignoring the case.
-int Utility::strcasecmp( const char *s1, const char *s2 )
-{
-  return ::strcasecmp( s1, s2 );
-}
-
-
 // Writes a vector.
 int Utility::writev(int socket, const iovec *vector, size_t count )
 {
